<?php

/*
  Plugin Name: Like Gate
  Plugin URI: http://webweb.ca/site/products/like-gate/
  Description: Like Gate allows you to reveal some hidden/secret content when the user likes the article. Therefore increasing the likeness of your articles
  Tags: wordpress,wp,plugins,facebook,fb,like,likes,socia media,social media,viral,viral content
  Version: 1.0.1
  Author: Svetoslav Marinov (Slavi)
  Author URI: http://orbisius.com
  License: GPL v2
 */

/*
  Copyright 2011-2020 Svetoslav Marinov (slavi@slavi.biz)

  This program ais free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

// we can be called from the test script
if (empty($_ENV['WEBWEB_WP_LIKE_GATE_TEST'])) {
    // Make sure we don't expose any info if called directly
    if (!function_exists('add_action')) {
        echo "Hi there!  I'm just a plugin, not much I can do when called directly.";
        exit;
    }

	$webweb_wp_like_gate_obj = WebWeb_WP_LikeGate::get_instance();

    add_action('init', array($webweb_wp_like_gate_obj, 'init'));

    register_activation_hook(__FILE__, array($webweb_wp_like_gate_obj, 'on_activate'));
    register_deactivation_hook(__FILE__, array($webweb_wp_like_gate_obj, 'on_deactivate'));
    register_uninstall_hook(__FILE__, array($webweb_wp_like_gate_obj, 'on_uninstall'));

    include_once(WEBWEB_WP_LIKE_GATE_BASE_DIR . '/zzz_dashboard_widgets.php');
}

class WebWeb_WP_LikeGate {
    private $log = 1;
    private static $instance = null; // singleton
    private $site_url = null; // filled in later
    private $plugin_url = null; // filled in later
    private $plugin_settings_key = null; // filled in later
    private $plugin_dir_name = null; // filled in later
    private $plugin_data_dir = null; // plugin data directory. for reports and data storing. filled in later
    private $plugin_name = 'Like Gate'; //
    private $plugin_id_str = 'like-gate'; //
    private $plugin_business_sandbox = false; // sandbox or live ???
    private $plugin_business_email_sandbox = 'seller_1264288169_biz@slavi.biz'; // used for paypal payments
    private $plugin_business_email = 'billing@WebWeb.ca'; // used for paypal payments
    private $plugin_business_ipn = 'http://webweb.ca/wp/hosted/payment/ipn.php'; // used for paypal IPN payments
    //private $plugin_business_status_url = 'http://localhost/wp/hosted/payment/status.php'; // used after paypal TXN to to avoid warning of non-ssl return urls
    private $plugin_business_status_url = 'https://secure.webweb.ca/webweb.ca/wp/hosted/payment/status.php'; // used after paypal TXN to to avoid warning of non-ssl return urls
    private $plugin_support_email = 'help@WebWeb.ca'; //
    private $plugin_support_link = 'http://miniads.ca/widgets/contact/profile/like-gate?height=200&width=500&description=Please enter your enquiry below.'; //
    private $plugin_admin_url_prefix = null; // filled in later
    private $plugin_home_page = 'http://webweb.ca/site/products/like-gate/';
    private $plugin_tinymce_name = 'wwwplikegate'; // if you change it update the tinymce/editor_plugin.js and reminify the .min.js file.
    private $plugin_cron_hook = __CLASS__;
    private $plugin_cron_freq = 'daily';
    private $plugin_default_opts = array(
        'status' => 0,
    );

	private $app_title = 'Like Gate: increase the likeness of your articles!';
	private $plugin_description = 'Like Gate allows you to reveal some hidden/secret content when the user likes the article. Therefore increasing the likeness of your articles';

    // can't be instantiated; just using get_instance
    private function __construct() {

    }

    /**
     * handles the singleton
     */
    function get_instance() {
		if (is_null(self::$instance)) {
			$cls = __CLASS__;
			$inst = new $cls;

			$site_url = get_settings('siteurl');

			$inst->site_url = $site_url;
			$inst->plugin_dir_name = basename(dirname(__FILE__)); // e.g. wp-command-center; this can change e.g. a 123 can be appended if such folder exist
			$inst->plugin_data_dir = dirname(__FILE__) . '/data';
			$inst->plugin_url = $site_url . '/wp-content/plugins/' . $inst->plugin_dir_name . '/';
			$inst->plugin_settings_key = $inst->plugin_id_str . '_settings';
            $inst->plugin_support_link .= '&css_file=' . urlencode(get_bloginfo('stylesheet_url'));

			// not sure if this will work here
			// Use when develing to trigger cron sooner e.g. every 3 mins.
			//add_filter('cron_schedules', array($webweb_wp_like_gate_obj, 'define_cron_frequencies'));
			//add_filter('cron_schedules', array($inst, 'define_cron_frequencies'));
			//$inst->plugin_cron_freq = $inst->plugin_id_str . '3min';

            $inst->plugin_admin_url_prefix = $site_url . '/wp-admin/admin.php?page=' . $inst->plugin_dir_name;

			define('WEBWEB_WP_LIKE_GATE_BASE_DIR', dirname(__FILE__)); // e.g. // htdocs/wordpress/wp-content/plugins/wp-command-center
			define('WEBWEB_WP_LIKE_GATE_DIR_NAME', $inst->plugin_dir_name);

			if ($inst->log) {
				ini_set('log_errors', 1);
				ini_set('error_log', $inst->plugin_data_dir . '/error.log');
			}

			add_action('plugins_loaded', array($inst, 'init'), 100);

            self::$instance = $inst;
        }

		return self::$instance;
	}

    public function __clone() {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    public function __wakeup() {
        trigger_error('Unserializing is not allowed.', E_USER_ERROR);
    }

    /**
     * handles the init
     */
    function init() {
        global $wpdb;

        // TODO: gather stats for the likes via cron ?
        //add_action($this->plugin_cron_hook, array($this, 'process_cron'));

        if (is_admin()) {
            // Administration menus
            add_action('admin_menu', array($this, 'administration_menu'));
            add_action('admin_init', array($this, 'add_buttons'));
            add_action('admin_init', array($this, 'register_settings'));
            add_action('admin_notices', array($this, 'notices'));

            wp_register_style($this->plugin_dir_name, $this->plugin_url . 'css/main.css', false, 0.2);
            wp_enqueue_style($this->plugin_dir_name);
        } else {
            if (!is_feed()) {
                add_action('wp_head', array($this, 'add_meta_header'));
                wp_enqueue_script('jquery');

                // The short code is has a closing *tag* e.g. [tag]...[/tag] so normal tag partse won't work
                //add_shortcode($this->plugin_id_str, array($this, 'parse_short_code'));
                //add_filter('the_content', array($this, 'parse_short_code'), 10000); // run last to check fb container and other stuff are added

                add_action('wp', array($this, 'setup_output_handler'), 10000, 0);
            }
        }
    }

    /**
     * Setups the callback for the output handler
     *
     * @param void
     */
    function setup_output_handler() {
        if (is_feed() || is_admin()) {
            return;
        }

        ob_start(array($this, 'parse_output'));
    }

    /**
     * Parse the whole content (wp doesn't still allow that).
     * We need to add some attributes in the <html> tag even before the <head>...tag
     *
     * @param string $buffer
     * @return string
     */
    function parse_output($buffer) {
        $buffer = $this->check_for_missing_namespaces($buffer);
        $buffer = $this->parse_short_code($buffer);

        return $buffer;
    }

    /**
     * searches and replaces the short code
	 * [like-gate] ..... [/like-gate] everything in between is hidden from non-likers
     */
    function parse_short_code($buffer) {
        global $post;
        $post_id = $post->ID;
        $app_id = '109282182482812'; // hardcoded my like gate app

        // https://developers.facebook.com/docs/reference/javascript/
        // Adding a Channel File greatly improves the performance of the JS SDK by addressing issues with cross-domain communication in certain browsers.
        $channel_url = $this->get('plugin_url') . 'channel.html';

        $like_gate_buffer = '';

        $opts = $this->get_options();

        if ((stripos($buffer, '[like-gate]') !== false)
				&& (stripos($buffer, '[/like-gate]') !== false)) {

            // Plugin is inactive let's remove the tags
            if (empty($opts['status'])) {
                $buffer = preg_replace('#\[like-gate\](.*?)\[/like-gate\]#si', '', $buffer, $matches);

                return $buffer;
            }

            // encrypt the text enclosed into the like gate short code
            // we want the replacement to start from within the <body> because there could be header/meta fields that match our
            // short code.
			$buffer = preg_replace('#(<body[^>]*>.*?)\[like-gate\](.*?)\[/like-gate\]#sie', '$this->encrypt_text("\\2", "\\1")', $buffer);

            // is it added already?
            if (stripos($buffer, '<div id="fb-root"></div>') === false) {
                $like_gate_buffer .= '<div id="fb-root"></div>';
            }

            // is it added already?
            // 2013-02-20: we'll add
            /*if (stripos($buffer, 'connect.facebook.net/en_US/all.js') === false) {
                $like_gate_buffer .= '<script src="http://connect.facebook.net/en_US/all.js#appId=109282182482812&amp;xfbml=1"></script>';
            }*/

            if (stripos($buffer, 'LIKE_GATE_OUTPUT') === false) { // only once
                $cookie_domain = $_SERVER['HTTP_HOST'];
                $cookie_domain = preg_replace('#^(blog|www|store|secure|ssl|test)\.#si', '', $cookie_domain); // rm www.
                $cookie_domain = '.' . $cookie_domain; // leading dot

                $like_gate_buffer .= <<<JS_EOF
<script>
    // LIKE_GATE_OUTPUT
    var cookie_domain = '$cookie_domain';

    // Load the SDK's source Asynchronously
    // Note that the debug version is being actively developed and might
    // contain some type checks that are overly strict.
    // Please report such bugs using the bugs tool.
    (function(d, debug){
       var js, id = 'facebook-jssdk', ref = d.getElementsByTagName('script')[0];
       if (d.getElementById(id)) {return;}
       js = d.createElement('script'); js.id = id; js.async = true;
       js.src = "//connect.facebook.net/en_US/all" + (debug ? "/debug" : "") + ".js";
       ref.parentNode.insertBefore(js, ref);
     }(document, /*debug*/ false));

    if (typeof jQuery.cookie == 'undefined') {
        /**
         * Cookie plugin
         *
         * Copyright (c) 2006 Klaus Hartl (stilbuero.de)
         * Dual licensed under the MIT and GPL licenses:
         * http://www.opensource.org/licenses/mit-license.php
         * http://www.gnu.org/licenses/gpl.html
         *
         */

        /**
         * Create a cookie with the given name and value and other optional parameters.
         *
         * @example $.cookie('the_cookie', 'the_value');
         * @desc Set the value of a cookie.
         * @example $.cookie('the_cookie', 'the_value', { expires: 7, path: '/', domain: 'jquery.com', secure: true });
         * @desc Create a cookie with all available options.
         * @example $.cookie('the_cookie', 'the_value');
         * @desc Create a session cookie.
         * @example $.cookie('the_cookie', null);
         * @desc Delete a cookie by passing null as value. Keep in mind that you have to use the same path and domain
         *       used when the cookie was set.
         *
         * @param String name The name of the cookie.
         * @param String value The value of the cookie.
         * @param Object options An object literal containing key/value pairs to provide optional cookie attributes.
         * @option Number|Date expires Either an integer specifying the expiration date from now on in days or a Date object.
         *                             If a negative value is specified (e.g. a date in the past), the cookie will be deleted.
         *                             If set to null or omitted, the cookie will be a session cookie and will not be retained
         *                             when the the browser exits.
         * @option String path The value of the path atribute of the cookie (default: path of page that created the cookie).
         * @option String domain The value of the domain attribute of the cookie (default: domain of page that created the cookie).
         * @option Boolean secure If true, the secure attribute of the cookie will be set and the cookie transmission will
         *                        require a secure protocol (like HTTPS).
         * @type undefined
         *
         * @name $.cookie
         * @cat Plugins/Cookie
         * @author Klaus Hartl/klaus.hartl@stilbuero.de
         */

        /**
         * Get the value of a cookie with the given name.
         *
         * @example $.cookie('the_cookie');
         * @desc Get the value of a cookie.
         *
         * @param String name The name of the cookie.
         * @return The value of the cookie.
         * @type String
         *
         * @name $.cookie
         * @cat Plugins/Cookie
         * @author Klaus Hartl/klaus.hartl@stilbuero.de
         */
        jQuery.cookie = function(name, value, options) {
            if (typeof value != 'undefined') { // name and value given, set cookie
                options = options || {};
                if (value === null) {
                    value = '';
                    options.expires = -1;
                }
                var expires = '';
                if (options.expires && (typeof options.expires == 'number' || options.expires.toUTCString)) {
                    var date;
                    if (typeof options.expires == 'number') {
                        date = new Date();
                        date.setTime(date.getTime() + (options.expires * 24 * 60 * 60 * 1000));
                    } else {
                        date = options.expires;
                    }
                    expires = '; expires=' + date.toUTCString(); // use expires attribute, max-age is not supported by IE
                }
                // CAUTION: Needed to parenthesize options.path and options.domain
                // in the following expressions, otherwise they evaluate to undefined
                // in the packed version for some reason...
                var path = options.path ? '; path=' + (options.path) : '';
                var domain = options.domain ? '; domain=' + (options.domain) : '';
                var secure = options.secure ? '; secure' : '';
                document.cookie = [name, '=', encodeURIComponent(value), expires, path, domain, secure].join('');
            } else { // only name given, get cookie
                var cookieValue = null;
                if (document.cookie && document.cookie != '') {
                    var cookies = document.cookie.split(';');
                    for (var i = 0; i < cookies.length; i++) {
                        var cookie = jQuery.trim(cookies[i]);
                        // Does this cookie string begin with the name we want?
                        if (cookie.substring(0, name.length + 1) == (name + '=')) {
                            cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                            break;
                        }
                    }
                }
                return cookieValue;
            }
        };
     }

     // credits: phpjs, Jonas Raoni Soares Silva (http://www.jsfromhell.com), Ates Goral (http://magnetiq.com), Onno Marsman, Rafał Kukawski (http://blog.kukawski.pl)
     function like_gate_decrypt(str, pwd) {
        return (str + '').replace(/[a-z]/gi, function (s) {
            return String.fromCharCode(s.charCodeAt(0) + (s.toLowerCase() < 'n' ? 13 : -13));
        });
     }

     // handles like and unlike events
     function like_gate_handle_event(pars) {
       pars = pars || {};

       if (pars.event == 'unlike') {
            jQuery('.like-gate-result').hide('slow').html('');
            jQuery.cookie('like_gate_lp_{$post_id}', null, { path: '/' });

            return true;
       }

       // , domain: '$cookie_domain' // chrome doesn't like the domain !?! doesn't delete the cookie.
       jQuery.cookie('like_gate_lp_{$post_id}', 1, { expires: 365, path: '/' });

       var decrypted_hidden = like_gate_decrypt(jQuery('.like-gate').html());
       jQuery('.like-gate-result').html(decrypted_hidden).show('slow');
     }

     jQuery(document).ready(function() {
        var like_status = jQuery.cookie('like_gate_lp_{$post_id}');

        // let's reveal if the user has already liked the page.
        if (like_status > 0) {
            like_gate_handle_event({event:'like'});
        }
     });

     function wp_like_gate_setup_callbacks() {
        // handles like
		FB.Event.subscribe('edge.create', function(href, widget) {
           like_gate_handle_event({
                event: 'like',
                url : href,
                widget: widget,

                '' : ''
           });
		});

		 // handles unlike, hide the hidden content
		FB.Event.subscribe('edge.remove', function(href, widget) {
           like_gate_handle_event({event:'unlike'});
		});
     }

	 // %%LIKE_GATE_FB_INIT%%
</script>
JS_EOF;
            } // /LIKE_GATE_OUTPUT

            $fb_init = <<<JS_EOF
    // Like Gate FB INIT: https://developers.facebook.com/docs/reference/javascript/
        window.fbAsyncInit = function() {
                FB.init({
                    appId      : '$app_id', // App ID from the App Dashboard
                    channelUrl : '$channel_url', // Channel File for x-domain communication
                    status     : true, // check the login status upon init?
                    cookie     : true, // set sessions cookies to allow your server to access the session?
                    xfbml      : true  // parse XFBML tags on this page?
                });

                wp_like_gate_setup_callbacks();
            };
    // Like Gate FB INIT END
JS_EOF;
            // fb jssdk hasn't been instantiated by another fb plugin
            if (stripos($buffer, 'window.fbAsyncInit') === false) {
                $like_gate_buffer = preg_replace('#\/+\s*%%LIKE_GATE_FB_INIT%%#si', $fb_init, $like_gate_buffer);
            } else {
                // inject a call to wp_like_gate_setup_callbacks() right after existing FB.init method.
                $existing_fb_init_regex = '#(window\.fbAsyncInit.*FB.init.*?}\s*\)\s*;)#si';
                $buffer = preg_replace($existing_fb_init_regex, "\\1\n\t// like-gate-auto-injected\n" . 'wp_like_gate_setup_callbacks();', $buffer);
            }

            // prepend the closing body tag
            $buffer = preg_replace('#</body>#si', $like_gate_buffer . "\n\\0", $buffer);

            //$like_gate_buffer
		} // like gate short codes

		return $buffer;
    }

    /**
     * encrypts the hidden text so it's hard to see
     */
    function encrypt_text($buffer, $buffer_preceeding_buffer = '') {
        $style = 'style="display:none;"';

        global $post;
        $post_url = get_permalink($post->ID); // the user will like this

        $like_box =<<<LIKE_BOX
<!-- like-gate-result -->
    <div id='like-gate-result' class='like-gate-result' $style></div>
<!-- /like-gate-result -->

<div class='like_gate_like_container' post_url="$post_url" post_id='{$post->ID}'>
        <fb:like href="$post_url" layout="standard" show-faces="false" width="450" action="like" colorscheme="light"></fb:like>
</div>
LIKE_BOX;

        //$buffer = "<!-- like-gate:id:{$post->ID}" . urlencode($buffer) . "<!-- /enc -->";
        $buffer = $like_box . "<!-- like-gate-secret id:{$post->ID} -->\n<div $style class='like-gate like-gate-secret' post_id='{$post->ID}' post_url='$post_url'>"
            . str_rot13($buffer) . "\n</div>\n<!-- /like-gate-secret -->\n"; // . var_export($post, 1);

        $buffer = $buffer_preceeding_buffer . $buffer;

		return $buffer;
    }

    /**
     * Handles the plugin activation. Setup cron and set default configs
     */
    function install_cron() {
        $when = mktime(23, 30, 0, date('m'), date('d'), date('Y'));
        $when = time();

        wp_schedule_event($when, $this->plugin_cron_freq, $this->plugin_cron_hook);
    }

    /**
     * Handles the plugin activation. Setup cron and set default configs
     */
    function uninstall_cron() {
        wp_clear_scheduled_hook($this->plugin_cron_hook);
    }

    /**
     * checks if WP has installed the hook
     * @return bool
     */
    function is_cron_scheduled() {
        $status = wp_get_schedule($this->plugin_cron_hook);

        return $status !== false;
    }

    /**
     * Handles the plugin activation. Setup cron and set default configs
     * This code was left from another plugin of mine and should not be here.
     * let's clean old cron stuff if any.
     */
    function on_activate() {
        if ($this->is_cron_scheduled()) {
            $this->uninstall_cron();
        }
    }

    /**
     * Handles the plugin deactivation. Remove cron and set default configs
     */
    function on_deactivate() {
        //$opts['status'] = 0;
        //$this->set_options($opts);
        $this->uninstall_cron();
    }

    /**
     * Handles the plugin uninstallation. remove cron and set default configs
     */
    function on_uninstall() {
        delete_option($this->plugin_settings_key);
        $this->uninstall_cron();
    }

    /**
     * Allows access to some private vars
     * @param str $var
     */
    public function get($var) {
        if (isset($this->$var) /* && (strpos($var, 'plugin') !== false) */) {
            return $this->$var;
        }
    }

    /**
     * Allows access to some private vars
     * @param str $var
     */
    public function generate_newsletter_box() {
        $file = WEBWEB_WP_LIKE_GATE_BASE_DIR . '/zzz_newsletter_box.html';

        $buffer = WebWeb_WP_LikeGateUtil::read($file);

        wp_get_current_user();
        global $current_user;
        $user_email = $current_user->user_email;

        $replace_vars = array(
            '%%PLUGIN_URL%%' => $this->get('plugin_url'),
            '%%USER_EMAIL%%' => $user_email,
        );

        $buffer = str_replace(array_keys($replace_vars), array_values($replace_vars), $buffer);

        return $buffer;
    }

    /**
     * Allows access to some private vars
     * @param str $var
     */
    public function generate_donate_box() {
        $msg = '';
        $file = WEBWEB_WP_LIKE_GATE_BASE_DIR . '/zzz_donate_box.html';

        if (!empty($_REQUEST['error'])) {
            $msg = $this->message('There was a problem with the payment.');
        }

        if (!empty($_REQUEST['ok'])) {
            $msg = $this->message('Thank you so much!', 1);
        }

        $return_url = WebWeb_WP_LikeGateUtil::add_url_params($this->get('plugin_business_status_url'), array(
            'r' => $this->get('plugin_admin_url_prefix') . '/menu.dashboard.php&ok=1', // paypal de/escapes
            'status' => 1,
        ));

        $cancel_url = WebWeb_WP_LikeGateUtil::add_url_params($this->get('plugin_business_status_url'), array(
            'r' => $this->get('plugin_admin_url_prefix') . '/menu.dashboard.php&error=1', //
            'status' => 0,
        ));

        $replace_vars = array(
            '%%MSG%%' => $msg,
            '%%AMOUNT%%' => '2.99',
            '%%BUSINESS_EMAIL%%' => $this->plugin_business_email,
            '%%ITEM_NAME%%' => $this->plugin_name . ' Donation',
            '%%ITEM_NAME_REGULARLY%%' => $this->plugin_name . ' Donation (regularly)',
            '%%PLUGIN_URL%%' => $this->get('plugin_url'),
            '%%CUSTOM%%' => http_build_query(array('site_url' => $this->site_url, 'product_name' => $this->plugin_id_str)),
            '%%NOTIFY_URL%%' => $this->get('plugin_business_ipn'),
            '%%RETURN_URL%%' => $return_url,
            '%%CANCEL_URL%%' => $cancel_url,
        );

        // Let's switch the Sandbox settings.
        if ($this->plugin_business_sandbox) {
            $replace_vars['paypal.com'] = 'sandbox.paypal.com';
            $replace_vars['%%BUSINESS_EMAIL%%'] = $this->plugin_business_email_sandbox;
        }

        $buffer = WebWeb_WP_LikeGateUtil::read($file);
        $buffer = str_replace(array_keys($replace_vars), array_values($replace_vars), $buffer);

        return $buffer;
    }

    /**
     * gets current options and return the default ones if not exist
     * @param void
     * @return array
     */
    function get_options() {
        $opts = get_option($this->plugin_settings_key);
        $opts = empty($opts) ? array() : (array) $opts;

        // if we've introduced a new default key/value it'll show up.
        $opts = array_merge($this->plugin_default_opts, $opts);

        if (empty($opts['notification_email'])) {
            $opts['notification_email'] = get_option('admin_email');
        }

        return $opts;
    }

    /**
     * Updates options but it merges them unless $override is set to 1
     * that way we could just update one variable of the settings.
     */
    function set_options($opts = array(), $override = 0) {
        if (!$override) {
            $old_opts = $this->get_options();
            $opts = array_merge($old_opts, $opts);
        }

        update_option($this->plugin_settings_key, $opts);

        return $opts;
    }

    /**
     * This is what the plugin admins will see when they click on the main menu.
     * @var string
     */
    private $plugin_landing_tab = '/menu.dashboard.php';

    /**
     * Adds the settings in the admin menu
     */
    public function administration_menu() {
        // Settings > Like Gate
        add_options_page(__($this->plugin_name, "WEBWEB_WP_PARTNER_WATCHER"), __($this->plugin_name, "WEBWEB_WP_PARTNER_WATCHER"), 'manage_options', __FILE__, array($this, 'options'));

        // Main page
        add_menu_page(__($this->plugin_name, $this->plugin_dir_name), __($this->plugin_name, $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.dashboard.php', null, $this->plugin_url . '/images/icon.png');

        // Sub Pages
        add_submenu_page($this->plugin_dir_name . '/' . $this->plugin_landing_tab, __('Dashboard', $this->plugin_dir_name), __('Dashboard', $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.dashboard.php');
        add_submenu_page($this->plugin_dir_name . '/' . $this->plugin_landing_tab, __('Settings', $this->plugin_dir_name), __('Settings', $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.settings.php');
        add_submenu_page($this->plugin_dir_name . '/' . $this->plugin_landing_tab, __('FAQ', $this->plugin_dir_name), __('FAQ', $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.faq.php');
        add_submenu_page($this->plugin_dir_name . '/' . $this->plugin_landing_tab, __('Help', $this->plugin_dir_name), __('Help', $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.support.php');
        //add_submenu_page($this->plugin_dir_name . '/' . $this->plugin_landing_tab, __('Contact', $this->plugin_dir_name), __('Contact', $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.contact.php');
        //add_submenu_page($this->plugin_dir_name . '/' . $this->plugin_landing_tab, __('About', $this->plugin_dir_name), __('About', $this->plugin_dir_name), 'manage_options', $this->plugin_dir_name . '/menu.about.php');

        // when plugins are show add a settings link near my plugin for a quick access to the settings page.
        add_filter('plugin_action_links', array($this, 'add_plugin_settings_link'), 10, 2);
    }

    /**
     * Outputs some options info. No save for now.
     */
    function options() {
		$webweb_wp_like_gate_obj = WebWeb_WP_LikeGate::get_instance();
        $opts = get_option('settings');

        include_once(WEBWEB_WP_LIKE_GATE_BASE_DIR . '/menu.settings.php');
    }

    /**
     * Sets the setting variables
     */
    function register_settings() { // whitelist options
        register_setting($this->plugin_dir_name, $this->plugin_settings_key);
    }

    // Add the ? settings link in Plugins page very good
    function add_plugin_settings_link($links, $file) {
        if ($file == plugin_basename(__FILE__)) {
            $settings_link = '<a href="http://club.orbisius.com/support/" target="_blank">' . (__("Support", "WEBWEB_WP_PARTNER_WATCHER")) . '</a>';
                array_unshift($links, $settings_link);

            $settings_link = '<a href="options-general.php?page='
                    . dirname(plugin_basename(__FILE__)) . '/' . basename(__FILE__) . '">' . (__("Settings", "WEBWEB_WP_PARTNER_WATCHER")) . '</a>';
            array_unshift($links, $settings_link);
        }

        return $links;
    }

    function add_meta_header() {
        printf("\n" . '<meta name="generator" content="Powered by ' . $this->plugin_name . ' (' . $this->plugin_home_page . ') " />' . PHP_EOL);
    }

    // kept for future use if necessary

    /**
     * Adds buttons only for RichText mode
     * @return void
     */
    function add_buttons() {
        // Don't bother doing this stuff if the current user lacks permissions
        if (!current_user_can('edit_posts') && !current_user_can('edit_pages')) {
            return;
        }

        // Add only in Rich Editor mode
        if (get_user_option('rich_editing') == 'true') {
            // add the button for wp2.5 in a new way
            add_filter("mce_external_plugins", array($this, "add_tinymce_plugin"), 5);
            add_filter('mce_buttons', array(&$this, 'register_button'), 5);
        }
    }

    // used to insert button in wordpress 2.5x editor
    function register_button($buttons) {
        array_push($buttons, "separator", $this->plugin_tinymce_name);

        return $buttons;
    }

    // Load the TinyMCE plugin : editor_plugin.js (wp2.5)
    function add_tinymce_plugin($plugin_array) {
        $plugin_array[$this->plugin_tinymce_name] = $this->plugin_url . 'tinymce/editor_plugin.min.js';

        return $plugin_array;
    }

    /**
     * Checks if WP simpple shopping cart is installed.
     */
    function notices() {
        $opts = $this->get_options();

        if (empty($opts['status'])) {
            echo $this->message($this->plugin_name . " is currently disabled. Please, enable it from "
                    ."<a href='{$this->plugin_admin_url_prefix}/menu.settings.php'> {$this->plugin_name} &gt; Settings</a>");
        }
    }

    /**
     * Outputs a message (adds some paragraphs)
     */
    function message($msg, $status = 0) {
        $id = $this->plugin_id_str;
        $cls = empty($status) ? 'app_error fade' : 'app_success';

        $str = <<<MSG_EOF
<div id='$id-notice' class='app_message_box $cls'><p><strong>$msg</strong></p></div>
MSG_EOF;
        return $str;
    }

    /**
     * a simple status message, no formatting except color
     */
    function msg($msg, $status = 0) {
        $id = $this->plugin_id_str;
        $cls = empty($status) ? 'app_error' : 'app_success';

        $str = <<<MSG_EOF
<div id='$id-notice' class='$cls'><strong>$msg</strong></div>
MSG_EOF;
        return $str;
    }

    /**
     * a simple status message, no formatting except color, simpler than its brothers
     */
    function m($msg, $status = 0) {
        $cls = empty($status) ? 'app_error' : 'app_success';

        $str = <<<MSG_EOF
<span class='$cls'>$msg</span>
MSG_EOF;
        return $str;
    }

    /**
     * Adds missing namespaces because the like will not show up in IE 6,7,8 if they are not set. the header must contain some tags so IE allows tags starting with <fb:... >
     * @param string $matched_str
     * @return string
     */
    public function check_for_missing_namespaces($buffer) {
        // Adds missing namespaces because the like will not show up in IE 6,7,8 if they are not set
        $buffer = preg_replace('#<html([^>]*)>#sie', "WebWeb_WP_LikeGateUtil::add_missing_namespaces('\\1')", $buffer);

        return $buffer;
    }
}

class WebWeb_WP_LikeGateUtil {
    // options for read/write methods.
    const FILE_APPEND = 1;
    const UNSERIALIZE_DATA = 2;
    const SERIALIZE_DATA = 3;

    /**
     * Gets the content from the body, removes the comments, scripts
     * Credits: http://php.net/manual/en/function.strip-tags.phpm /  http://networking.ringofsaturn.com/Web/removetags.php
     * @param string $buffer
     * @string string $buffer
     */
    public static function html2text($buffer = '') {
        // we care only about the body so it must be beautiful.
        $buffer = preg_replace('#.*<body[^>]*>(.*?)</body>.*#si', '\\1', $buffer);
        $buffer = preg_replace('#<script[^>]*>.*?</script>#si', '', $buffer);
        $buffer = preg_replace('#<style[^>]*>.*?</style>#siU', '', $buffer);
//        $buffer = preg_replace('@<style[^>]*>.*?</style>@siU', '', $buffer); // Strip style tags properly
        $buffer = preg_replace('#<[a-zA-Z\/][^>]*>#si', ' ', $buffer); // Strip out HTML tags  OR '@<[\/\!]*?[^<>]*\>@si',
        $buffer = preg_replace('@<![\s\S]*?--[ \t\n\r]*>@', '', $buffer); // Strip multi-line comments including CDATA
        $buffer = preg_replace('#[\t\ ]+#si', ' ', $buffer); // replace just one space
        $buffer = preg_replace('#[\n\r]+#si', "\n", $buffer); // replace just one space
        //$buffer = preg_replace('#(\s)+#si', '\\1', $buffer); // replace just one space
        $buffer = preg_replace('#^\s*|\s*$#si', '', $buffer);

        return $buffer;
    }

    /**
     * Gets the content from the body, removes the comments, scripts
     *
     * @param string $buffer
     * @param array $keywords
     * @return array - for now it returns hits; there could be some more complicated results in the future so it's better as an array
     */
    public static function match($buffer = '', $keywords = array()) {
        $status_arr['hits'] = 0;

        foreach ($keywords as $keyword) {
            $cnt = preg_match('#\b' . preg_quote($keyword) . '\b#si', $buffer);

            if ($cnt) {
                $status_arr['hits']++; // total hits
                $status_arr['matches'][$keyword] = array('keyword' => $keyword, 'hits' => $cnt,); // kwd hits
            }
        }

        return $status_arr;
    }

    /**
     * @desc write function using flock
     *
     * @param string $vars
     * @param string $buffer
     * @param int $append
     * @return bool
     */
    public static function write($file, $buffer = '', $option = null) {
        $buff = false;
        $tries = 0;
        $handle = '';

        $write_mod = 'wb';

        if ($option == self::SERIALIZE_DATA) {
            $buffer = serialize($buffer);
        } elseif ($option == self::FILE_APPEND) {
            $write_mod = 'ab';
        }

        if (($handle = @fopen($file, $write_mod))
                && flock($handle, LOCK_EX)) {
            // lock obtained
            if (fwrite($handle, $buffer) !== false) {
                @fclose($handle);
                return true;
            }
        }

        return false;
    }

    /**
     * @desc read function using flock
     *
     * @param string $vars
     * @param string $buffer
     * @param int $option whether to unserialize the data
     * @return mixed : string/data struct
     */
    public static function read($file, $option = null) {
        $buff = false;
        $read_mod = "rb";
        $tries = 0;
        $handle = false;

        if (($handle = @fopen($file, $read_mod))
                && (flock($handle, LOCK_EX))) { //  | LOCK_NB - let's block; we want everything saved
            $buff = @fread($handle, filesize($file));
            @fclose($handle);
        }

        if ($option == self::UNSERIALIZE_DATA) {
            $buff = unserialize($buff);
        }

        return $buff;
    }

    /**
     *
     * Appends a parameter to an url; uses '?' or '&'
     * It's the reverse of parse_str().
     *
     * @param string $url
     * @param array $params
     * @return string
     */
    public static function add_url_params($url, $params = array()) {
        $str = '';

        $params = (array) $params;

        if (empty($params)) {
            return $url;
        }

        $query_start = (strpos($url, '?') === false) ? '?' : '&';

        foreach ($params as $key => $value) {
            $str .= ( strlen($str) < 1) ? $query_start : '&';
            $str .= rawurlencode($key) . '=' . rawurlencode($value);
        }

        $str = $url . $str;

        return $str;
    }

    // generates HTML select
    public static function html_select($name = '', $options = array(), $sel = null, $attr = '') {
        $html = "\n" . '<select name="' . $name . '" ' . $attr . '>' . "\n";

        foreach ($options as $key => $label) {
            $selected = $sel == $key ? ' selected="selected"' : '';
            $html .= "\t<option value='$key' $selected>$label</option>\n";
        }

        $html .= '</select>';
        $html .= "\n";

        return $html;
    }

    // generates status msg
    public static function msg($msg = '', $status = 0) {
        $cls = empty($status) ? 'error' : 'success';
        $cls = $status == 2 ? 'notice' : $cls;

        $msg = "<p class='status_wrapper'><div class=\"status_msg $cls\">$msg</div></p>";

        return $msg;
    }

    /**
     * Adds missing namespaces because the like will not show up in IE 6,7,8 if they are not set
     * @param string $matched_str
     * @return string
     */
    public static function add_missing_namespaces($matched_str) {
        $og = 'xmlns:og="http://opengraphprotocol.org/schema/"';
        $fb = 'xmlns:fb="http://www.facebook.com/2008/fbml"';

        if (stripos($matched_str, 'xmlns:og') === false) {
            $matched_str .= ' ' . $og;
        }

        if (stripos($matched_str, 'xmlns:fb') === false) {
            $matched_str .= ' ' . $fb;
        }

        $matched_str = '<html' . stripslashes($matched_str) . '>';

        return $matched_str;
    }
}

class WebWeb_WP_LikeGateCrawler {

    private $user_agent = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0) Gecko/20100101 Firefox/6.0";
    private $error = null;
    private $buffer = null;

    function __construct() {
        ini_set('user_agent', $this->user_agent);
    }

    /**
     * Error(s) from the last request
     *
     * @return string
     */
    function getError() {
        return $this->error;
    }

    // checks if buffer is gzip encoded
    function is_gziped($buffer) {
        return (strcmp(substr($buffer, 0, 8), "\x1f\x8b\x08\x00\x00\x00\x00\x00") === 0) ? true : false;
    }

    /*
      henryk at ploetzli dot ch
      15-Feb-2002 04:28
      http://php.online.bg/manual/hu/function.gzencode.php
     */

    function gzdecode($string) {
        if (!function_exists('gzinflate')) {
            return false;
        }

        $string = substr($string, 10);
        return gzinflate($string);
    }

    /**
     * Fetches a url and saves the data into an instance variable. The returned status is whether the request was successful.
     *
     * @param string $url
     * @return bool
     */
    function fetch($url) {
        $ok = 0;
        $buffer = '';

        $url = trim($url);

        if (!preg_match("@^(?:ht|f)tps?://@si", $url)) {
            $url = "http://" . $url;
        }

        // try #1 cURL
        // http://fr.php.net/manual/en/function.fopen.php
        if (empty($ok)) {
            if (function_exists("curl_init") && extension_loaded('curl')) {
                $ch = curl_init();

                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_HEADER, 0);
                curl_setopt($ch, CURLOPT_HTTPHEADER, array('Accept-Encoding: gzip'));
                curl_setopt($ch, CURLOPT_TIMEOUT, 45);
                curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
                curl_setopt($ch, CURLOPT_MAXREDIRS, 5); /* Max redirection to follow */
                curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
                curl_setopt($ch, CURLOPT_USERAGENT, $this->user_agent);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

                /* curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC ) ; // in the future pwd protected dirs
                  curl_setopt($ch, CURLOPT_USERPWD, "username:password"); */ //  http://php.net/manual/en/function.curl-setopt.php

                $string = curl_exec($ch);
                $curl_res = curl_error($ch);

                curl_close($ch);

                if (empty($curl_res) && strlen($string)) {
                    if ($this->is_gziped($string)) {
                        $string = $this->gzdecode($string);
                    }

                    $this->buffer = $string;

                    return 1;
                } else {
                    $this->error = $curl_res;
                    return 0;
                }
            }
        } // empty ok*/
        // try #2 file_get_contents
        if (empty($ok)) {
            $buffer = @file_get_contents($url);

            if (!empty($buffer)) {
                $this->buffer = $buffer;
                return 1;
            }
        }

        // try #3 fopen
        if (empty($ok) && preg_match("@1|on@si", ini_get("allow_url_fopen"))) {
            $fp = @fopen($url, "r");

            if (!empty($fp)) {
                $in = '';

                while (!feof($fp)) {
                    $in .= fgets($fp, 8192);
                }

                @fclose($fp);
                $buffer = $in;

                if (!empty($buffer)) {
                    $this->buffer = $buffer;
                    return 1;
                }
            }
        }

        return 0;
    }

    function get_content() {
        return $this->buffer;
    }
}